@@ -1,29 +1,17 @@
-# A parallel version of cluster::clusGap, modified from there
+# A parallel version of cluster::clusGap, modified from the R package 'cluster'
+# Maechler, M., Rousseeuw, P., Struyf, A., Hubert, M., Hornik, K.(2021).
+# cluster: Cluster Analysis Basics and Extensions. R package version 2.1.2.
 #' @importFrom stats dist runif var
 #' @importFrom parallel detectCores mclapply
 #' @importFrom cluster maxSE
-clusGapP <- function(x, FUNcluster, K.max, B = 100, d.power = 1, spaceH0 = c(
-                       "scaledPCA",
-                       "original"
-                     ), MaxCores = 2048, nProc = 2, ...) {
+clusGapP <- function(x, FUNcluster, K.max, B = 100, d.power = 1, spaceH0 = c("scaledPCA",
+  "original"), nProc, ...) {
   stopifnot(is.function(FUNcluster), length(dim(x)) == 2, K.max >= 2, (n <- nrow(x)) >=
     1, ncol(x) >= 1)
   if (B != (B. <- as.integer(B)) || (B <- B.) <= 0) {
     stop("'B' has to be a positive integer")
   }
 
-  if (!missing(nProc)) {
-    nProc <- nProc
-  } else {
-    chk <- Sys.getenv("_R_CHECK_LIMIT_CORES_", "")
-    if (nzchar(chk) && chk == "TRUE") {
-      num_workers <- 2L
-    } else {
-      num_workers <- parallel::detectCores()
-    }
-    nProc <- min(num_workers - 1, MaxCores)
-  }
-
   cl. <- match.call()
   if (is.data.frame(x)) {
     x <- as.matrix(x)
@@ -37,7 +25,7 @@ clusGapP <- function(x, FUNcluster, K.max, B = 100, d.power = 1, spaceH0 = c(
     }
     0.5 * sum(vapply(split(ii, clus), function(I) {
       xs <- X[I, , drop = FALSE]
-      sum(dist(xs)^d.power / nrow(xs))
+      sum(dist(xs)^d.power/nrow(xs))
     }, 0))
   }
   logW <- E.logW <- SE.sim <- numeric(K.max)
@@ -47,15 +35,11 @@ clusGapP <- function(x, FUNcluster, K.max, B = 100, d.power = 1, spaceH0 = c(
   spaceH0 <- match.arg(spaceH0)
   xs <- scale(x, center = TRUE, scale = FALSE)
   m.x <- rep(attr(xs, "scaled:center"), each = n)
-  switch(spaceH0,
-    scaledPCA = {
-      V.sx <- svd(xs, nu = 0)$v
-      xs <- xs %*% V.sx
-    },
-    original = {
-    },
-    stop("invalid 'spaceH0':", spaceH0)
-  )
+  switch(spaceH0, scaledPCA = {
+    V.sx <- svd(xs, nu = 0)$v
+    xs <- xs %*% V.sx
+  }, original = {
+  }, stop("invalid 'spaceH0':", spaceH0))
   rng.x1 <- apply(xs, 2L, range)
   logWks <- matrix(0, B, K.max)
   logWks1 <- matrix(0, 1, K.max)
@@ -65,12 +49,8 @@ clusGapP <- function(x, FUNcluster, K.max, B = 100, d.power = 1, spaceH0 = c(
   if (nProc > 1 & !Sys.info()[["sysname"]] == "Windows") {
     BootGap <- parallel::mclapply(list.of.bootstraps, function(b) {
       z1 <- apply(rng.x1, 2, function(M, nn) runif(nn, min = M[1], max = M[2]),
-        nn = n
-      )
-      z <- switch(spaceH0,
-        scaledPCA = tcrossprod(z1, V.sx),
-        original = z1
-      ) +
+        nn = n)
+      z <- switch(spaceH0, scaledPCA = tcrossprod(z1, V.sx), original = z1) +
         m.x
       for (k in 1:K.max) {
         logWks1[, k] <- log(W.k(z, k))
@@ -80,12 +60,8 @@ clusGapP <- function(x, FUNcluster, K.max, B = 100, d.power = 1, spaceH0 = c(
   } else {
     BootGap <- lapply(list.of.bootstraps, function(b) {
       z1 <- apply(rng.x1, 2, function(M, nn) runif(nn, min = M[1], max = M[2]),
-        nn = n
-      )
-      z <- switch(spaceH0,
-        scaledPCA = tcrossprod(z1, V.sx),
-        original = z1
-      ) +
+        nn = n)
+      z <- switch(spaceH0, scaledPCA = tcrossprod(z1, V.sx), original = z1) +
         m.x
       for (k in 1:K.max) {
         logWks1[, k] <- log(W.k(z, k))
@@ -96,9 +72,7 @@ clusGapP <- function(x, FUNcluster, K.max, B = 100, d.power = 1, spaceH0 = c(
   logWks <- matrix(unlist(BootGap), ncol = K.max, nrow = B, byrow = T)
 
   E.logW <- colMeans(logWks)
-  SE.sim <- sqrt((1 + 1 / B) * apply(logWks, 2, var))
-  structure(class = "clusGap", list(Tab = cbind(logW, E.logW,
-    gap = E.logW - logW,
-    SE.sim
-  ), call = cl., spaceH0 = spaceH0, n = n, B = B, FUNcluster = FUNcluster))
-}
\ No newline at end of file
+  SE.sim <- sqrt((1 + 1/B) * apply(logWks, 2, var))
+  structure(class = "clusGap", list(Tab = cbind(logW, E.logW, gap = E.logW - logW,
+    SE.sim), call = cl., spaceH0 = spaceH0, n = n, B = B, FUNcluster = FUNcluster))
+}
